require 'shellwords'

require 'kitchen/login_command'
require 'kitchen/shell_out'
require 'kitchen/transport/base'
require 'thor/util'

require 'kitchen/helper/k8s'

module Kitchen
  # A Transport which uses kubectl to execute commands and transfer files.
  module Transport
    # Kubernetes transport for Kitchen. Uses kubectl exec.
    class K8s < Kitchen::Transport::Base
      # All configuration options can be found in the Driver class.
      def connection(state, &block)
        Connection.new(
          pod_id: state[:pod_id],
          namespace: config[:namespace],
          binary: config[:binary]
        ).tap do |conn|
          yield(conn) if block
        end
      end
    end

    # Connection class for Kitchen
    class Connection < Kitchen::Transport::Base::Connection
      include ShellOut
      include KitchenK8s::Helper

      # (see Base::Connection#execute)
      def execute(command)
        return if command.nil?
        run_command(kubectl_command('exec', '--tty', '--container=default', options[:pod_id], '--', *Shellwords.split(wrap_command(command))))
      end

      # (see Base::Connection#upload)
      def upload(locals, remote)
        return if locals.empty?
        locals.each do |local|
          full_remote = "#{options[:namespace]}/#{options[:pod_id]}:#{remote}"
          run_command(kubectl_command('cp', local, full_remote, '-c', 'default'))
        end
      end

      protected

      def wrap_command(cmd)
        cmd.match(/\Ash\s\-c/) ? cmd : Util.wrap_command(cmd.gsub('\'', "'\\\\''"))
      end
    end
  end
end
