require 'mixlib/shellout'

require 'train'

module Train::Transports
  # Hack for verifier
  class K8sHack < Train.plugin(1)
    name 'k8s_hack'

    include_options Train::Extras::CommandWrapper
    option :pod, required: true
    option :container, default: nil
    option :kubectl_path, default: 'kubectl'
    option :context, default: nil

    def connection(state = {})
      opts = merge_options(options, state || {})
      validate_options(opts)
      opts[:logger] ||= logger
      unless @connection && @connection_opts == opts
        @connection ||= Connection.new(opts)
        @connection_opts = opts.dup
      end
      @connection
    end

    # Base
    class Connection < BaseConnection
      if Gem::Requirement.create('< 0.30').satisfied_by?(Gem::Version.create(Train::VERSION))
        # The API for a connection changed a lot in 0.30, this is a compat shim.
        def os
          @os ||= OSCommon.new(self, family: 'unix')
        end

        def file(path)
          @files[path] ||= file_via_connection(path)
        end

        def run_command(cmd)
          run_command_via_connection(cmd)
        end
      end

      def file_via_connection(path)
        defined?(Train::File::Remote::Linux) ? Train::File::Remote::Linux.new(self, path) : LinuxFile.new(self, path)
      end

      def run_command_via_connection(cmd)
        kubectl_cmd = [options[:kubectl_path], 'exec']
        kubectl_cmd.concat(['--context', options[:context]]) if options[:context]
        kubectl_cmd.concat(['--container', options[:container]]) if options[:container]
        kubectl_cmd.concat([options[:pod], '--', '/bin/sh', '-c', cmd])

        so = Mixlib::ShellOut.new(kubectl_cmd, logger: logger)
        so.run_command
        if so.error?
          # Trim the "command terminated with exit code N" line from the end
          # of the stderr content.
          so.stderr.gsub!(/command terminated with exit code #{so.exitstatus}\n\Z/, '')
        end
        CommandResult.new(so.stdout, so.stderr, so.exitstatus)
      end

      def uri
        if options[:container]
          "kubernetes://#{options[:pod]}/#{options[:container]}"
        else
          "kubernetes://#{options[:pod]}"
        end
      end
    end
  end
end
